/*!
 * © 2022 David Vines
 *
 * domainOfTheAncients is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * domainOfTheAncients is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with domainOfTheAncients. If not, see https://www.gnu.org/licenses/gpl-2.0.html.
 */
/* globals Util, JumpShipOrder, JumpToAndColonisePlanetShipOrder, HoldAndColoniseShipOrder, HoldPositionShipOrder */
/* exported ShipOrderReport */
let ShipOrderReport = {
	_changeButtons: [],
	showShipReports: function(player,map) {
		let ships = document.getElementById("ships");
		ShipOrderReport._changeButtons.length = 0;
		for(let ship of player.ships) {
			const newrow = ships.insertRow();
			const namecell = document.createElement("td"); namecell.append(ship.name);
			const loccell = document.createElement("td"); loccell.append(ship.location);
			const repcell = document.createElement("td"); repcell.append(ship.report);
			ship.resetOrder(); // Reset to hold position
			const ordercell = document.createElement("td"); ordercell.append("Hold position");
			const changecell = document.createElement("td");
			if (ship.getJumpRange() > 0) {
				const which = {player: player, map: map, ship: ship, ordercell: ordercell, coloniseTickBox: null };
				const changeJumpButton = document.createElement("button"); changeJumpButton.append("Change Jump Target...");
				ShipOrderReport._changeButtons.push(changeJumpButton);
				changecell.append(changeJumpButton);
				if (ship.getColonyImprovement()) {
					const coloniseTickBox = document.createElement("input"); coloniseTickBox.type = "checkbox";
					coloniseTickBox.checked = ship.order instanceof JumpToAndColonisePlanetShipOrder;
					coloniseTickBox.onchange = Util.captureExceptionFunction(ShipOrderReport._coloniseChange,which);
					changecell.append(document.createElement("br"));
					changecell.append(coloniseTickBox);
					changecell.append("Colonise On Arrival");
					which.coloniseTickBox = coloniseTickBox;
				}
				changeJumpButton.onclick = Util.captureExceptionFunction(ShipOrderReport._changeJump,which);
				newrow.append(namecell,loccell,repcell,ordercell,changecell);
			} else {
				changecell.append("Not Jump Capable");
			}
			newrow.append(namecell,loccell,repcell,ordercell,changecell);
		}
	},
	_changeJump: function(which) {
		// First disable all of the change buttons
		for(let button of ShipOrderReport._changeButtons) {
			button.disabled = true;
		}
		Util.replaceText("shipinstructionsShipName",which.ship.name);
		document.getElementById("shipinstructions").classList.remove("hidden");
		document.getElementById("cancelJumpChange").onclick = Util.captureExceptionFunction(ShipOrderReport._cancelJumpTarget,which);

		const location = which.ship.location;
		const jumpRange = which.ship.getJumpRange();
		which.map.computeCommsGrid(which.player);
		for(let row=1; row<21; row++) {
			for(let column of "ABCDEFGHIJKLMNOP") {
				const name = column+row;
				const hexelement = document.getElementById(name);
				if (Map.range(location,name) <= jumpRange && which.map.hex(name).onGrid) {
					const args = Object.assign({target: name},which);
					hexelement.onclick = Util.captureExceptionFunction(ShipOrderReport._selectJumpTarget,args);
				} else {
					hexelement.classList.add("greyout");
				}
			}
		}
	},
	_selectJumpTarget: function(which) {
		if (which.coloniseTickBox && which.coloniseTickBox.checked && which.target == which.ship.location) {
			which.ship.order = HoldAndColoniseShipOrder.SINGLETON;
		} else if (which.coloniseTickBox && which.coloniseTickBox.checked) {
			which.ship.order = new JumpToAndColonisePlanetShipOrder(which.target);
		} else if (which.target == which.ship.location) {
			which.ship.resetOrder();
		} else {
			which.ship.order = new JumpShipOrder(which.target);
		}
		ShipOrderReport._finishJumpTarget(which);
	},
	_coloniseChange: function(which) {
		if (which.coloniseTickBox.checked && which.ship.order instanceof HoldPositionShipOrder) {
			which.ship.order = HoldAndColoniseShipOrder.SINGLETON;
		} else if (!which.coloniseTickBox.checked && which.ship.order instanceof HoldAndColoniseShipOrder) {
			which.ship.resetOrder();
		} else if (which.coloniseTickBox.checked && which.ship.order instanceof JumpShipOrder) {
			which.ship.order = new JumpToAndColonisePlanetShipOrder(which.ship.order.target);
		}
		else if (!which.coloniseTickBox.checked && which.ship.order instanceof JumpToAndColonisePlanetShipOrder) {
			which.ship.order = new JumpShipOrder(which.ship.order.target);
		}
		ShipOrderReport._fillInOrders(which);
	},
	_fillInOrders: function(which) {
		which.ordercell.replaceChildren();
		which.ordercell.append(which.ship.order.toString());
	},
	_cancelJumpTarget: function(which) {
		if (which.coloniseTickBox && which.coloniseTickBox.checked) {
			which.ship.order = new JumpToAndColonisePlanetShipOrder(which.ship.location);
		} else {
			which.ship.resetOrder();
		}
		ShipOrderReport._finishJumpTarget(which);
	},
	_finishJumpTarget: function(which) {
		ShipOrderReport._fillInOrders(which);
		document.getElementById("shipinstructions").classList.add("hidden");
		const cancel = document.getElementById("cancelJumpChange");
		cancel.onclick = null;
		for(let row=1; row<21; row++) {
			for(let column of "ABCDEFGHIJKLMNOP") {
				const name = column+row;
				const hexelement = document.getElementById(name);
				hexelement.onclick = null;
				hexelement.classList.remove("greyout");
			}
		}
		for(let button of ShipOrderReport._changeButtons) {
			button.disabled = false;
		}
	}

};